#! /usr/bin/env python
# -*- coding: utf-8 -*-
import calendar
from datetime import date
from datetime import timedelta

from DateRanger.utils import get_quarter
from DateRanger.utils import get_monthrange
from DateRanger.exceptions import InvalidDateRange


class DateFrame(object):

    """
    An object for operations(get difference/yield date range) of date range.
    All methods do not include end_date.
    """

    def __init__(self, start_date, end_date):
        """
        Argus:
            start_date - the start date
            end_date - the end date
        """
        if start_date > end_date:
            raise InvalidDateRange()

        self.start_date = start_date
        self.end_date = end_date

    def get_timedelta(self):
        """
        Get timedelta of `self.end_date - self.start_date`
        """
        return self.end_date - self.start_date

    def days(self):
        """
        Calcualte the difference in days.
        """
        return int(self.get_timedelta().days)

    def each_day(self):
        """
        Yield each day between start_date and end_date.
        """
        if self.days() == 0:
            yield self.start_date
        else:
            for n in range(self.days()+1):
                yield self.start_date + timedelta(n)

    def get_weekdelta(self):
        """
        Simplify this question by counting weeks between monday1 and monday2.

        monday1 - monday of the week of self.start_date
        monday2 - monday of the week of self.end_date
        """
        monday1 = (self.start_date - timedelta(days=self.start_date.weekday()))
        monday2 = (self.end_date - timedelta(days=self.end_date.weekday()))
        # if self.start_date and self.end_date in the same week.
        if (monday2 - timedelta(days=1)) == self.start_date:
            return timedelta(days=0)
        else:
            return monday2 - monday1

    def weeks(self):
        """
        Return date difference in months.
        """
        return int(self.get_weekdelta().days / 7)

    def each_week(self):
        """
        Yield each week between self.start_date and self.end_date
        """
        if self.weeks() == 0:
            yield (self.end_date - timedelta(days=6), self.end_date)
        else:
            dow = self.start_date.weekday()
            start = (self.start_date - timedelta(days=dow+1))
            for n in range(self.weeks()+1):
                yield (start, start + timedelta(days=6))
                start = start + timedelta(days=7)

    def get_monthdelta(self):
        """
        Get month delta.
        """
        yeardelta = self.get_yeardelta()
        if yeardelta == 0:
            return self.end_date.month - self.start_date.month

        monthdelta = (13 - self.start_date.month) + \
            ((yeardelta - 1) * 12) + self.end_date.month
        return monthdelta

    def months(self):
        """
        Calcualte the difference in months.
        """
        return self.get_monthdelta()

    def each_month(self):
        """
        Yield each month between self.start_date and self.end_date
        """
        start = date(self.start_date.year, self.start_date.month, 1)
        if self.months() == 0:
            days = calendar.monthrange(start.year, start.month)[1]
            yield (start, date(start.year, start.month, days))

        for n in range(self.months()):
            days = calendar.monthrange(start.year, start.month)[1]
            end = date(start.year, start.month, days)
            yield (start, end)
            start = end + timedelta(days=1)

    def get_quarterdelta(self):
        """
        Calcualte date difference in quraters.
        """
        squarter = get_quarter(self.start_date.month)
        equarter = get_quarter(self.end_date.month)
        yeardelta = self.get_yeardelta()
        if yeardelta == 0:
            return equarter - squarter

        quarterdelta = (4 - squarter) + ((yeardelta - 1) * 4) + equarter
        return quarterdelta

    def quarters(self):
        """
        Return date difference in quraters.
        """
        return self.get_quarterdelta()

    def each_quarter(self):
        """
        Yield each quarter.
        """
        quarter = get_quarter(self.start_date.month)
        start_month, end_month = get_monthrange(quarter)
        start = date(self.start_date.year, start_month, 1)
        if self.quarters() == 0:
            days = calendar.monthrange(start.year, start.month + 2)[1]
            end = date(start.year, start.month + 2, days)
            yield (start, end)

        for n in range(self.quarters()):
            days = calendar.monthrange(start.year, start.month + 2)[1]
            end = date(start.year, start.month + 2, days)
            yield (start, end)
            start = end + timedelta(days=1)

    def get_yeardelta(self):
        """
        Calcualte date difference in years.
        """
        return int(self.end_date.year) - int(self.start_date.year)

    def years(self):
        """
        Return date difference in years.
        """
        return self.get_yeardelta()

    def each_year(self):
        """
        Yield each year.
        """
        for n in range(self.start_date.year, self.end_date.year + 1):
            yield (date(n, 1, 1), date(n, 12, 31))

    def get_range(self):
        """
        Return a tuple that contains self.start_date and self.end_date
        """
        return (self.start_date, self.end_date)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from DateRanger.exceptions import InvalidMonth
from DateRanger.exceptions import InvalidQuarter


def get_quarter(month):
    """
    Determine a month belongs to which quarter.

    Argus:
        month - 1 ~ 12 (Integer)
    """
    if month not in range(1, 13):
        raise InvalidMonth()

    quarter = 4
    if month in (1, 2, 3): quarter = 1
    if month in (4, 5, 6): quarter = 2
    if month in (7, 8, 9): quarter = 3
    return quarter


def get_monthrange(quarter):
    """
    Get start and end month of a quarter.

    Argus:
        quarter - 1 ~ 4 (Integer)
    """
    if quarter not in range(1, 5):
        raise InvalidQuarter()

    monthranges = ((1, 3), (4, 6), (7, 9), (10, 12))
    return monthranges[quarter-1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import calendar
from datetime import date
from datetime import timedelta

from DateRanger.utils import get_quarter
from DateRanger.utils import get_monthrange
from DateRanger.objects import DateFrame
from DateRanger.exceptions import InvalidDateRange
from DateRanger.exceptions import InvalidQuarter


class DateRanger(object):

    """
    A class for getting common bussiness date ranges.
    """

    def __init__(self, base_date=None):
        """
        Argus:
            base_date - the base day. Example: date(2009, 11, 1)
        """
        self.set_base_date(base_date)

    def set_base_date(self, base_date=None):
        """
        Set base date.

        Argus:
            base_date - Example: date(2009, 11, 1)

        Note:
            self.b____, 'b' means 'base'
        """
        self.bdate = base_date or date.today()
        self.bmonth = self.bdate.month
        self.byear = self.bdate.year
        self.bquarter = get_quarter(self.bmonth)

    def base_day(self):
        """
        Get the DateRange of self.bdate.
        """
        return DateFrame(self.bdate, self.bdate)

    def relative_day(self, days=0):
        """
        Calcuate a relative date from self.bdate.
        """
        rday = self.bdate + timedelta(days=days)
        return (rday, rday)

    def prev_day(self, days=1):
        """
        Get the DateRange that n days before self.bdate.

        Argus:
            days - n days ago
        """
        ndays = days * -1
        start, end = self.relative_day(days=ndays)
        return DateFrame(start, end)

    def next_day(self, days=1):
        """
        Get the DateRange that n days after self.bdate.

        Argus:
            days - next n days
        """
        start, end = self.relative_day(days=days)
        return DateFrame(start, end)

    def get_week_range(self, base_date):
        """
        Find the first/last day of the week for the given day.
        Weeks start on Sunday and end on Saturday.

        Argus:
            base_date - any date
        """
        start =  base_date - timedelta(days=base_date.weekday()+1)
        end = start + timedelta(days=6)
        return (start, end)

    def base_week(self):
        """
        Get DateRange of the week that contains self.bdate.
        """
        start, end = self.get_week_range(self.bdate)
        return DateFrame(start, end)

    def relative_week(self, weeks=0):
        """
        Calcuate a relative week range from self.bdate.
        """
        _, end_date = self.base_week().get_range()
        start, end = self.get_week_range(end_date + timedelta(days=7*weeks))
        return (start, end)

    def prev_week(self, weeks=1):
        """
        Get the DateRange that n weeks before self.bdate.

        Argus:
            weeks - n week ago
        """
        nweeks = weeks * -1
        start, end = self.relative_week(weeks=nweeks)
        return DateFrame(start, end)

    def next_week(self, weeks=1):
        """
        Get the DateRange that n weeks after self.bdate.

        Argus:
            weeks - next n weeks
        """
        start, end = self.relative_week(weeks=weeks)
        return DateFrame(start, end)

    def get_month_range(self, year, month):
        """
        Get the first and last day of the given month in given year.

        Args:
            year
            month
        """
        days = calendar.monthrange(year, month)[1]
        start = date(year, month, 1)
        end = date(year, month, days)
        return (start, end)

    def relative_month(self, months=0):
        """
        Calcuate a relative month range from self.bdate.
        """
        month_sum = self.bmonth + months
        if month_sum < 0:
            back_months = abs(month_sum)
            yeardelta = ((back_months // 12) + 1) * -1
            month = 12 - (back_months % 12)
        elif month_sum == 0:
            yeardelta = -1
            month = 12
        elif month_sum <= 12:
            yeardelta = 0
            month = month_sum
        else:
            yeardelta = month_sum // 12
            month = month_sum % 12
        year = self.byear + yeardelta
        start, end = self.get_month_range(year, month)
        return (start, end)

    def base_month(self):
        """
        Get the DateRange of the month that contains self.bdate
        """
        year, month = self.byear, self.bmonth
        start, end = self.get_month_range(year, month)
        return DateFrame(start, end)

    def prev_month(self, months=1):
        """
        Get the DateRange that n months before self.bdate.

        Argus:
            months - n months ago
        """
        nmonths = months * -1
        start, end = self.relative_month(months=nmonths)
        return DateFrame(start, end)

    def next_month(self, months=1):
        """
        Get the DateRange that n months after self.bdate.

        Argus:
            months - next n months
        """
        start, end = self.relative_month(months=months)
        return DateFrame(start, end)

    def get_quarter_range(self, year, quarter):
        """
        Get time range with specific year and quarter.
        """
        if quarter not in (1, 2, 3, 4):
            raise InvalidQuarter()

        start_month, end_month = get_monthrange(quarter)
        days = calendar.monthrange(year, end_month)[1]
        start = date(year, start_month, 1)
        end = date(year, end_month, days)
        return start, end

    def relative_quarter(self, quarters=0):
        """
        Calcuate a relative quarters range from self.bdate.
        """
        quarter_sum = self.bquarter + quarters
        if quarter_sum < 0:
            back_quarters = abs(quarter_sum)
            yeardelta = ((back_quarters // 4) + 1) * -1
            quarter = 4 - (back_quarters % 4)
        elif quarter_sum == 0:
            yeardelta = -1
            quarter = 4
        elif quarter_sum <= 4:
            yeardelta = 0
            quarter = quarter_sum
        else:
            yeardelta = quarter_sum // 4
            quarter = quarter_sum % 4
        year = self.byear + yeardelta
        start, end = self.get_quarter_range(year, quarter)
        return (start, end)

    def base_quarter(self):
        """
        Get the DateRange of the quarter that contains self.bdate.
        """
        quarter = get_quarter(self.bmonth)
        start, end = self.get_quarter_range(self.byear, quarter)
        return DateFrame(start, end)

    def prev_quarter(self, quarters=1):
        """
        Get the DateRange that n quarters before self.bdate.

        Argus:
            quarters - n quarters ago
        """
        nquarters = quarters * -1
        start, end = self.relative_quarter(quarters=nquarters)
        return DateFrame(start, end)

    def next_quarter(self, quarters=1):
        """
        Get the DateRange that n quarters after self.bdate.

        Argus:
            quarters - next n quarters
        """
        start, end = self.relative_quarter(quarters=quarters)
        return DateFrame(start, end)

    def get_year_range(self, year):
        """
        Get time range of the year.
        """
        start = date(year, 1, 1)
        end = date(year, 12, 31)
        return (start, end)

    def relative_year(self, years=0):
        year = self.byear + years
        start, end = self.get_year_range(year)
        return (start, end)

    def base_year(self):
        """
        Get the DateRange of the year that contains self.bdate.
        """
        start, end = self.get_year_range(self.byear)
        return DateFrame(start, end)

    def prev_year(self, years=1):
        """
        Get the DateRange that n years before self.bdate.

        Argus:
            years - n years ago
        """
        nyears = years * -1
        start, end = self.relative_year(years=nyears)
        return DateFrame(start, end)

    def next_year(self, years=1):
        """
        Get the DateRange that n years after self.bdate.

        Argus:
            year - next n years
        """
        start, end = self.relative_year(years=years)
        return DateFrame(start, end)

    def from_date(self, from_date):
        """
        Return the DateRange from `from_date` to self.bdate

        Argus:
            from_date - Example: date(2015, 1, 1)
        """
        if from_date > self.bdate:
            raise InvalidDateRange()

        return DateFrame(from_date, self.bdate + timedelta(days=1))

    def to_date(self, to_date):
        """
        Return the DateRange from self.bdate to `to_date`

        Argus:
            to_date - Example: date(2015, 1, 1)
        """
        if to_date < self.bdate:
            raise InvalidDateRange()

        return DateFrame(self.bdate, to_date + timedelta(days=1))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from distutils.core import setup
from sys import version


if version < '2.7':
    raise NotImplementedError("DateRanger requires Python 2.7.* or above.")


setup(
    name='DateRanger',
    url='https://github.com/spitfire-sidra/DateRanger',
    version='0.3.0.alpha',
    author='Amo Chen',
    author_email='spitfire.sidra@gmail.com',
    packages=['DateRanger',],
    license='License.txt',
    description='Useful business date ranges',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Topic :: Utilities',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Programming Language :: Python :: Implementation :: CPython',
    ]
)
